package net.libertacasa.pubsh.web;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import org.keycloak.KeycloakPrincipal;
import org.keycloak.KeycloakSecurityContext;
import org.keycloak.adapters.KeycloakDeployment;
import org.keycloak.adapters.RefreshableKeycloakSecurityContext;
import org.keycloak.adapters.springsecurity.token.KeycloakAuthenticationToken;
import org.keycloak.representations.IDToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.example.demo.Docker;
import com.github.dockerjava.api.model.Container;
import com.github.dockerjava.api.model.Image;

@SpringBootApplication
@Controller
public class WebApplication {

	public static void main(String[] args) {
		SpringApplication.run(WebApplication.class, args);
	}

	@GetMapping("/")
	public String index() {
		return "homepage";
		
	}
	
	@GetMapping("/portal")
	public String portal(Model model) {
		KeycloakAuthenticationToken authentication = (KeycloakAuthenticationToken)
        SecurityContextHolder.getContext().getAuthentication();
        
		Principal principal = (Principal) authentication.getPrincipal();
		String username="";
		String email="";
		String attribute01="";
      
      if (principal instanceof KeycloakPrincipal) {
          KeycloakPrincipal<?> kPrincipal = (KeycloakPrincipal<?>) principal;
          IDToken token = kPrincipal.getKeycloakSecurityContext().getIdToken();
          System.out.println("Token: " + token);
          
          Map<String, Object> customClaims = token.getOtherClaims();
          
          if (customClaims.containsKey("email")) {
              email = String.valueOf(customClaims.get("email"));
          }        
          if (customClaims.containsKey("username")) {
        	  username = String.valueOf(customClaims.get("username"));
          }
          if (customClaims.containsKey("attribute01")) {
        	  attribute01 = String.valueOf(customClaims.get("attribute01"));
          }
          model.addAttribute("username", username);
          model.addAttribute("principalid", principal.getName());
          model.addAttribute("email", email);
          model.addAttribute("attribute01", attribute01);          
      }
      
      List<Image> images = Docker.getImages();
      //System.out.println(images);     
      List<Container> containers = Docker.getContainers();
      //System.out.println(containers);
      
      model.addAttribute("docker_images", images);
      model.addAttribute("docker_containers", containers);      
      
      return("portal");
	}
	
	@GetMapping(path = "/logout")
	public String logout(HttpServletRequest request) throws ServletException {
	  request.logout();
	  keycloakSessionLogout(request);
	  return "logout";
	 }
	
   private void keycloakSessionLogout(HttpServletRequest request){
      RefreshableKeycloakSecurityContext c = getKeycloakSecurityContext(request);
      KeycloakDeployment d = c.getDeployment();
      c.logout(d);
   }

   private RefreshableKeycloakSecurityContext getKeycloakSecurityContext(HttpServletRequest request){
	  return (RefreshableKeycloakSecurityContext) request.getAttribute(KeycloakSecurityContext.class.getName());
   }	
}
